#ifndef THINGS_THINGSEQUENCE_H
#define THINGS_THINGSEQUENCE_H
///
/// @file 
/// @version $Id$
/// @date $Date$
///

#include <vector>
#include "ThingInstanceBase.h"
#include "CompositeThing.h"

namespace Things
{

///
/// @brief
/// Class representing a collection of rule instances with a particular ordering
/// relationship between them.
///
/// An output from each rule will normally be an input to the next but
/// this is neither required nor enforced.
///
/// ThingSequenceNumbers work as follows:
/// * Each rule added to a sequence is assigned the next free
///   sequence number (i.e. they are in ascending order).
///
/// Contrast with the @link ThingSet@endlink container which assigns the same sequence number
/// to each rule.
///
/// @invariant
/// This class preserves the invariant that rules are ordered by ascending sequence number
/// @code sequenceNumber Y = this->getInitialSequenceNumber()
///       foreach X in this->begin(), this->end()
///          X->getSequenceNumber() > Y;
///          Y=X;
/// @code 
///
class ThingSequence: public CompositeThing
{
public:
   /// @name House-keeping
   /// @{

   ///
   /// @post
   /// @code this->empty() @endcode
   ///
   /// @post
   /// @code this->getInitialSequenceNumber() == initialSequenceNumber_ @endcode
   /// @code this->getStreamIdentifier() == streamIdentifier_ @endcode
   ///
   ThingSequence(const ProcessingNetwork& network,
                ThingSequenceNumber initialSequenceNumber_ = 0,
                ThingLabelFunctor& labelFunctor = NullLabelFunctor::getNullLabelFunctor(),
                const StreamIdentifier& streamIdentifier_ = "");

   virtual ~ThingSequence();

   /// @}
public:
   /// @name Attributes
   /// @{

   ///
   /// @brief
   /// The sequence number to assign to the first rule added to the container.
   ///
   ThingSequenceNumber getInitialSequenceNumber() const;

   /// @}
public:
   /// @name Attribute-like helpers
   /// @{

   ///
   /// @brief
   /// Return the highest used sequence number in the container
   /// or the initial sequence number if the container is empty.
   ///
   virtual ThingSequenceNumber getThingSequenceNumber() const;

   /// @}
public:
   /// @name Mutators
   /// @{

   ///
   /// @brief
   /// Update the sequence numbers in this collection.
   ///
   /// @post
   /// The initial sequence number is updated.
   /// @code this->getInitialSequenceNumber() == sequenceNumber_ @endcode
   ///
   /// @post
   /// The sequence numbers of all contained Thing and CompositeThing
   /// objects are updated as required.
   ///
   virtual void setThingSequenceNumber(ThingSequenceNumber sequenceNumber_);

   ///
   /// @brief
   /// Allow a visitor to process and possibly modify the ThingSequence.
   ///
   virtual void accept(NodeVisitor& visitor);

   ///
   /// @brief
   /// Allow a visitor to process the ThingSequence without modifying it.
   ///
   virtual void accept(ConstNodeVisitor& visitor) const;

   /// @}
public:
   /// @name Mutators
   /// @{

   ///
   /// @brief
   /// Add a rule or rule sequence to the sequence.
   ///
   /// @par warning
   /// The ThingInstanceBase must have a life-time at least as long as this ThingSequence.
   ///
   /// @pre
   /// @code !this->count(rule_) @endcode
   ///
   /// @pre
   /// The rule must be part of the same processing network as its container.
   /// @code &rule_.getProcessingNetwork() == &this->getProcessingNetwork() @endcode
   ///
   /// @post
   /// @code this->count(rule_) @endcode
   /// @code rule_.getSequenceNumber() == this->getSequenceNumber() @endcode
   ///
   virtual void addThing(ThingInstanceBase& rule_);

   /// @}
protected:
   /// @name State
   /// @{

   StreamIdentifier   streamIdentifier;
   ThingSequenceNumber initialSequenceNumber;

   /// @}
}; //endclass ThingSequence

} //end namespace

#endif
